package kr.nerv.antop.dao.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;

import javax.sql.DataSource;

import kr.nerv.antop.dao.UserDao;
import kr.nerv.antop.model.User;

public class UserDaoJdbc implements UserDao {
	protected DataSource dataSource;

	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	@Override
	public int create(User user) throws Exception {
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;

		int generatedKey = -1;

		try {
			conn = dataSource.getConnection();
			// query
			String sql = "insert into sample_user (name, age) values (?, ?)";

			pstmt = conn.prepareStatement(sql, PreparedStatement.RETURN_GENERATED_KEYS);
			pstmt.setString(1, user.getName());
			pstmt.setInt(2, user.getAge());

			// execute query
			pstmt.executeUpdate();
			
			rs = pstmt.getGeneratedKeys();
			if (rs.next()) {
				generatedKey = rs.getInt(1);
			}

			return generatedKey;
		} catch (Exception e) {
			throw e;
		} finally {
			release(rs, pstmt, conn);
		}
	}

	@Override
	public User findById(int seq) throws Exception {
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;

		User user = null;

		try {
			conn = dataSource.getConnection();

			// query
			String sql = "select * from sample_user where seq = ?";

			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, seq);

			rs = pstmt.executeQuery();

			if (rs.next()) {
				user = new User();
				user.setSeq(rs.getInt("seq"));
				user.setName(rs.getString("name"));
				user.setAge(rs.getInt("age"));
				user.setCreated(rs.getTimestamp("created"));
				user.setModified(rs.getTimestamp("modified"));
			}

			return user;
		} catch (Exception e) {
			throw e;
		} finally {
			release(rs, pstmt, conn);
		}
	}

	protected void release(ResultSet rs, PreparedStatement pstmt, Connection conn) {
		if (rs != null) {
			try {
				rs.close();
			} catch (Exception e) {
			}
		}
		if (pstmt != null) {
			try {
				pstmt.close();
			} catch (Exception e) {
			}
		}
		if (conn != null) {
			try {
				conn.close();
			} catch (Exception e) {
			}
		}
	}

}
