package kr.nerv.antop.dao.impl;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcDaoSupport;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;

import kr.nerv.antop.dao.UserDao;
import kr.nerv.antop.model.User;

public class UserDaoSpringJdbc extends NamedParameterJdbcDaoSupport implements UserDao {

	@Override
	public int create(User user) throws Exception {
		// query
		String sql = "insert into sample_user (name, age) values (:name, :age)";
		// parameter
		SqlParameterSource paramSource = new BeanPropertySqlParameterSource(user);
		// key holder
		KeyHolder generatedKeyHolder = new GeneratedKeyHolder();
		// execute query
		getNamedParameterJdbcTemplate().update(sql, paramSource, generatedKeyHolder);
		// auto increment
		return generatedKeyHolder.getKey().intValue();
	}

	// row mapper
	private UserRowMapper rowMapper = new UserRowMapper();

	@Override
	public User findById(int seq) throws Exception {
		// query
		String sql = "select * from sample_user where seq = :seq";
		// parameter
		Map<String, Integer> paramMap = new HashMap<String, Integer>();
		paramMap.put("seq", seq);
		
		try {
			// execute query
			return (User) getNamedParameterJdbcTemplate().queryForObject(sql, paramMap, rowMapper);
		} catch (EmptyResultDataAccessException e) {
			// EmptyResultDataAccessException 예외 발생시 null 리턴
			return null;
		}
	}

	class UserRowMapper implements RowMapper<User> {
		@Override
		public User mapRow(ResultSet rs, int rowIndex) throws SQLException {
			User user = new User();
			user.setSeq(rs.getInt("seq"));
			user.setName(rs.getString("name"));
			user.setAge(rs.getInt("age"));
			user.setCreated(rs.getTimestamp("created"));
			user.setModified(rs.getTimestamp("modified"));

			return user;
		}
	}

}
